#include <stdio.h>
#include <string.h>
#include <malloc.h>
#include <limits.h>
#include <stdlib.h>

#include "BLUEWAVE.H"
#include "fts001.h"

#define MAXAREAS 100
#define DIRSEP '/'	/* should be '\\' under *DOS, including NT */

typedef enum { false, true } bool;

static FILE *mix, *inf,*dat,*fti;
static INF_HEADER infhdr;

static struct area
{
  MIX_REC	mix;
  INF_AREA_INFO inf;
} areas[MAXAREAS];

static int ctAreas;

bool 
openfiles (const char *filename, char *bbsname)
{
  char fullname[_POSIX_PATH_MAX];
  char *pc;

  sprintf (fullname, "unzip -dwork %s", filename);
  if (system (fullname))
    return false;
  pc = strrchr (filename, DIRSEP );
  if (!pc)
    strcpy( bbsname, filename);
  else
    strcpy( bbsname, ++pc );	/* remove leading stuff */
  pc = strrchr (bbsname, '.');
  if (pc)
    *pc = 0;			/* truncate name after last dot */

  /* add default extensions and read files */
  sprintf (fullname, "work/%s.MIX", bbsname);
  if ((mix = fopen (fullname, "r")))
  {
    sprintf (fullname, "work/%s.INF", bbsname);
    if ((inf = fopen (fullname, "r")))
    {
      if (fread (&infhdr, sizeof infhdr, 1, inf) == 1)
      {
	struct area *pa;
	
	printf ("Packet from %s at %s\n", infhdr.sysop, infhdr.systemname);
	for (	ctAreas = 0, pa = areas;
	  	ctAreas < MAXAREAS && ! (feof (inf) || feof(mix));
		/* no automatic increment */
	    )
	{
	  if (fread (&pa->mix, sizeof (MIX_REC), 1, mix) != 1)
	    perror ("reading mix file");
	  else
	  {
	    if (pa->mix.totmsgs)
	    {
	      int differ;
	      
	      printf ("Area %s has %d msgs (%d personal)\t",
		  pa->mix.areanum, 
		  pa->mix.totmsgs,
		  pa->mix.numpers );

	      while( (differ = strcmp( pa->inf.areanum, pa->mix.areanum)
	      && ! feof( inf ) ) )
	      {
		if (fread (&pa->inf, sizeof (INF_AREA_INFO), 1, inf) != 1)
		  perror ("reading areas");
	      }
	      if ( differ )
		return false;
	      puts (pa->inf.title);
	      ctAreas++;
	      pa++;
	    }
	  }
	}
	sprintf (fullname, "work/%s.FTI", bbsname);
	if ((fti = fopen (fullname, "r")))
	{
	  sprintf (fullname, "work/%s.DAT", bbsname);
	  if ((dat = fopen (fullname, "r")))
	    return true;
	}
      }
    }
  }
  perror ("opening component files");
  return false;
}


static void exportarea( char *bbs, struct area *pa )
{
  static char *MsgBuffer=NULL;
  static int MsgSize=0;
  FTI_REC msg;
  int ctMsg;
  char pktname[_POSIX_PATH_MAX];
  FILE *fp;

  printf( "\nExporting %s\n", pa->inf.title );
  if ( ( fseek( fti, pa->mix.msghptr, SEEK_SET ) ) )
  {
    perror( "seeking fti" );
    exit(1);
  }
  
  sprintf( pktname, "%s.PKT", pa->inf.echotag );
  if ( (fp = fopen( pktname, "w" )) )
  {
    FTS1PKTHDR ph;
    memset( &ph, 0, sizeof ph );
    ph.Version = 2;
    ph.DestZone = ZONE;
    ph.DestNet = NET;
    ph.DestNode = NODE;
    ph.DestPoint = POINT;
    if ( fwrite( &ph, sizeof ph, 1, fp ) != 1 ) 
    {
      perror( "writing pkt header" );
      exit(1);
    }
    for ( ctMsg = pa->mix.totmsgs;
	  ctMsg && (!feof (fti)) && (fread (&msg, sizeof msg, 1, fti) == 1);
	  ctMsg--)
    {
      FTS1PKTMSG mh;
      memset( &mh, 0, sizeof mh );
      mh.Version = 2;
      if ( fwrite( &mh, sizeof mh, 1, fp ) != 1 ) 
      {
	perror( "writing msg header" );
	exit(1);
      }
      printf( "%s:\t%s\t", msg.date, msg.from );
      if (strlen( msg.from ) < 8) putchar( '\t' ); 
      if (strlen( msg.from ) < 16) putchar( '\t' ); 
      if (strlen( msg.from ) < 24) putchar( '\t' ); 
      printf( "to %s\n", msg.to );
      fwrite( msg.date,		strlen( msg.date ),		1, fp );
      if (strlen(msg.date) < 19)
	fwrite( ":00", 4, 1, fp );
      else
        fputc( 0, fp );
      fwrite( msg.to,		strlen( msg.to ) + 1,		1, fp );
      fwrite( msg.from, 	strlen( msg.from ) + 1,		1, fp );
      fwrite( msg.subject,	strlen( msg.subject ) + 1,	1, fp );
      if (MsgSize <= msg.msglength + 1)
      {
	MsgSize = msg.msglength + 1;
	if (!(MsgBuffer = realloc (MsgBuffer, MsgSize)))
	{
	  perror ("increasing message buffer");
	  exit( 1 );
	}
      }
      if (fseek (dat, msg.msgptr, SEEK_SET) == 0)
      {
	unsigned ReadLength;

	ReadLength = fread (MsgBuffer, 1, (unsigned) msg.msglength, dat);
	MsgBuffer[ReadLength] = '\0';
	fprintf( fp, "AREA:%s_%s\r", bbs, pa->inf.echotag );
	fwrite( MsgBuffer+1, ReadLength, 1, fp );
      }
      /* putchar( '.' ); */
    }
    fclose( fp );
  }
}

int
main (int argc, char **argv)
{
  int iArea, iarg;
  char bbsname[_POSIX_NAME_MAX];

  if (argc < 2)
  {
    printf ("Syntax: %s packetfilename[...]\n", argv[0]);
    printf ("\tThis program will read BlueWave(tm) downloaded mail,");
    printf ("\tand convert it to Fidonet technology mail packets.");
    return 1;
  }
  for (iarg = 1; iarg < argc; iarg++)
  {
    if (!openfiles (argv[iarg], bbsname))
      return 1;
    for (iArea = 0; iArea < ctAreas; iArea++)
      exportarea( bbsname, &areas[iArea] );
    fclose (fti);
    fclose (dat);
    fclose (inf);
    system( "rm -rf work" );
    remove( argv[iarg] );
  }
  return 0;
}
