/*
**  Structures for .PKT files
*/

const unsigned
  CW_2PLUS          =0x0001,                  /*Support for Type 2+ bundles*/
  CW_N2PLUS         =0x0100,                      /*Validation for CW_2PLUS*/
  PKTVERSION        =2;                     /*Current packet revision level*/

  /* Packet header, one per packet */

typedef struct
{
  unsigned short
    OrigNode,			/*Originating node */
    DestNode,			/*Destination node */
    Year,			/*Year created (1993-nnnn) */
    Month,			/*Month created (0-11) */
    Day,			/*Day created (1-31) */
    Hour,			/*Hour created (0-23) */
    Minute,			/*Minute created (0-59) */
    Second,			/*Second created (0-59) */
    Rate,			/*Baudrate (unused) */
    Version,			/*Packet version (2) */
    OrigNet,			/*Originating net */
    DestNet;			/*Destination net */

  unsigned char
    PCodeLo,			/*Product code (low-order) */
    PRevMajor;			/*Major version */

  char
    Password[8];		/*Packet password (not ASCIIZ) */

  unsigned short
    QMOrigZone,			/*Originating zone (QMail) */
    QMDestZone,			/*Destination zone (QMail) */
    AuxNet,			/*Auxillary net (unused) */
    CWValidate;			/*CapWord validation */

  unsigned char
    PCodeHi,			/*Product code (high-order) */
    PRevMinor;			/*Minor version */

  unsigned short
    CWCapWord,			/*Capabilities */
    OrigZone,			/*Originating zone (FSC-39) */
    DestZone,			/*Destination zone (FSC-39) */
    OrigPoint,			/*Originating point (FSC-39) */
    DestPoint;			/*Destination point (FSC-39) */

  long
    LongData;			/*Product dependent information */
}
__attribute__ ((packed)) FTS1PKTHDR;

  /* Packed message, none or more per packet */

typedef struct
{
  unsigned short
    Version,                                              /*Packet version (2)*/
    OrigNode,                                               /*Originating node*/
    DestNode,                                               /*Destination node*/
    OrigNet,                                                 /*Originating net*/
    DestNet,                                                 /*Destination net*/
    Attr,                                                        /*FTS1 status*/
    Cost;                                                    /*Cost of message*/
    /*
    **  The message header follows this but is of variable length ASCIIZ
    **  strings in the order: AsciiDate, ToUser, FromUser, Subject, and
    **  MsgText.
    */
}
__attribute__ ((packed)) FTS1PKTMSG;
